@@ -897,13 +897,15 @@ class TestGoFZF < TestBase
     # Custom script to use as $SHELL
     output = tempname + '.out'
     File.unlink output rescue nil
-    writelines tempname, ['#!/usr/bin/env bash', "echo $1 / $2 > #{output}"]
+    writelines tempname, ['#!/usr/bin/env bash', "echo $1 / $2 > #{output}", "sync"]
     system "chmod +x #{tempname}"
 
     tmux.send_keys "echo foo | SHELL=#{tempname} fzf --bind 'enter:execute:{}bar'", :Enter
     tmux.until { |lines| lines[-2].include? '1/1' }
     tmux.send_keys :Enter
+    tmux.until { |lines| lines[-2].include? '1/1' }
     tmux.send_keys 'C-c'
+    tmux.prepare
     assert_equal ['-c / "foo"bar'], File.readlines(output).map(&:chomp)
   ensure
     File.unlink output rescue nil
@@ -1137,10 +1139,22 @@ class TestGoFZF < TestBase
       `seq 10 | #{FZF} -f '1 | !1'`.lines.map(&:chomp)
   end
 
+  def test_hscroll_off
+    writelines tempname, ['=' * 10000 + '0123456789']
+    [0, 3, 6].each do |off|
+      tmux.prepare
+      tmux.send_keys "#{FZF} --hscroll-off=#{off} -q 0 < #{tempname}", :Enter
+      tmux.until { |lines| lines[-3].end_with?((0..off).to_a.join + '..') }
+      tmux.send_keys '9'
+      tmux.until { |lines| lines[-3].end_with? '789' }
+      tmux.send_keys :Enter
+    end
+  end
+
 private
   def writelines path, lines
     File.unlink path while File.exists? path
-    File.open(path, 'w') { |f| f << lines.join($/) }
+    File.open(path, 'w') { |f| f << lines.join($/) + $/ }
   end
 end
 
@@ -1355,23 +1369,42 @@ module CompletionTest
       tmux.send_keys 'C-L'
       lines[-1] == "kill #{pid}"
     end
-
-    def test_custom_completion
-      tmux.send_keys '_fzf_compgen_path() { echo "\$1"; seq 10; }', :Enter
-      tmux.prepare
-      tmux.send_keys 'ls /tmp/**', :Tab, pane: 0
-      tmux.until(1) { |lines| lines.item_count == 11 }
-      tmux.send_keys :BTab, :BTab, :BTab
-      tmux.until(1) { |lines| lines[-2].include? '(3)' }
-      tmux.send_keys :Enter
-      tmux.until do |lines|
-        tmux.send_keys 'C-L'
-        lines[-1] == "ls /tmp 1 2"
-      end
-    end
   ensure
     Process.kill 'KILL', pid.to_i rescue nil if pid
   end
+
+  def test_custom_completion
+    tmux.send_keys '_fzf_compgen_path() { echo "\$1"; seq 10; }', :Enter
+    tmux.prepare
+    tmux.send_keys 'ls /tmp/**', :Tab, pane: 0
+    tmux.until(1) { |lines| lines.item_count == 11 }
+    tmux.send_keys :BTab, :BTab, :BTab
+    tmux.until(1) { |lines| lines[-2].include? '(3)' }
+    tmux.send_keys :Enter
+    tmux.until do |lines|
+      tmux.send_keys 'C-L'
+      lines[-1] == "ls /tmp 1 2"
+    end
+  end
+
+  def test_unset_completion
+    tmux.send_keys 'export FOO=BAR', :Enter
+    tmux.prepare
+
+    # Using tmux
+    tmux.send_keys 'unset FOO**', :Tab, pane: 0
+    tmux.until(1) { |lines| lines[-2].include? ' 1/' }
+    tmux.send_keys :Enter
+    tmux.until { |lines| lines[-1] == 'unset FOO' }
+    tmux.send_keys 'C-c'
+
+    # FZF_TMUX=0
+    new_shell
+    tmux.send_keys 'unset FOO**', :Tab
+    tmux.until { |lines| lines[-2].include? ' 1/' }
+    tmux.send_keys :Enter
+    tmux.until { |lines| lines[-1] == 'unset FOO' }
+  end
 end
 
 class TestBash < TestBase
